//  Author : Christian Arild Str Andersen
//  			De Data Pty Ltd
//  Created : Mid 2003
//
//
// Use this class as you see fit at own risk
//
// AsciiString.cpp: implementation of the AsciiString class.
// this is just a small string class
// it was written in order to support simple 16bit string manipulation
// on a platform whit no support for the STL
//////////////////////////////////////////////////////////////////////

#include "AsciiString.h"


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

AsciiString::AsciiString()
{
	// always initilise the internal buffer to nul
	this->lpszData = NULL;
}

AsciiString::~AsciiString()
{
	// free up allocated memory
	ReleaseData();
}

ULONG AsciiString::length() const
{
	// returns the string lenght returns 0 if null ie. NULL == "" in lenght
	if(this->lpszData)
	{
		return strlen(this->lpszData);
	}
	return 0;
}


LPCSTR AsciiString::c_str() const
{
	// return pointer to buffer
	return lpszData;
}

VOID AsciiString::ReleaseData()
{
	// frees the date
	if(lpszData) delete lpszData;
	lpszData = NULL;
}


// self describing code
void AsciiString::empty()
{
	ReleaseData();
}

// here we have a few standard copy constructore

AsciiString::AsciiString(AsciiString &szClone)
{
	this->lpszData = NULL;
	ReleaseData();
	if(szClone.c_str())
	{
		lpszData = new CHAR[strlen(szClone.c_str()) + 1];
		if(lpszData)
		{
			memset(lpszData,0,strlen(szClone.c_str()) + 1);
			memcpy(lpszData,szClone.c_str(),strlen(szClone.c_str()));
		}
	}
}



AsciiString::AsciiString(LPSTR szClone)
{
	this->lpszData = NULL;
	ReleaseData();
	if(szClone)
	{
		lpszData = new CHAR[strlen(szClone) + 1];
		if(lpszData)
		{
			memset(lpszData,0,strlen(szClone) + 1);
			memcpy(lpszData,szClone,strlen(szClone));
		}
	}
}

AsciiString::AsciiString(LPWSTR lpwszClone)
{
	this->lpszData = NULL;
	ReleaseData();
	if(lpwszClone)
	{
		lpszData = new CHAR[wcslen(lpwszClone) + 1];
		if(lpszData)
		{
			memset(lpszData,0,wcslen(lpwszClone) + 1);
			WideCharToMultiByte(CP_ACP,NULL,lpwszClone,wcslen(lpwszClone),this->lpszData,wcslen(lpwszClone),NULL,NULL);
		}
	}
}

AsciiString::AsciiString(LPCSTR szClone)
{
	this->lpszData = NULL;
	ReleaseData();
	if(szClone)
	{
		lpszData = new CHAR[strlen(szClone) + 1];
		if(lpszData)
		{
			memset(lpszData,0,strlen(szClone) + 1);
			memcpy(lpszData,szClone,strlen(szClone));
		}
	}
}

AsciiString::AsciiString(LPCWSTR lpwszClone)
{
	this->lpszData = NULL;
	ReleaseData();
	if(lpwszClone)
	{
		lpszData = new CHAR[wcslen(lpwszClone) + 1];
		if(lpszData)
		{
			memset(lpszData,0,wcslen(lpwszClone) + 1);
			WideCharToMultiByte(CP_ACP,NULL,lpwszClone,wcslen(lpwszClone),this->lpszData,wcslen(lpwszClone),NULL,NULL);
		}
	}
}

AsciiString::AsciiString(CHAR aChar)
{
	this->lpszData = new CHAR[2];
	this->lpszData[0] = aChar;
	this->lpszData[1] = 0;
}

// now for the owerloaded operators


AsciiString   AsciiString::operator =(AsciiString & szClone)
{

	ReleaseData();
	if(szClone.c_str())
	{
		lpszData = new CHAR[strlen(szClone.c_str()) + 1];
		if(lpszData)
		{
			memset(lpszData,0,strlen(szClone.c_str()) + 1);
			memcpy(lpszData,szClone.c_str(),strlen(szClone.c_str()));
		}
	}
	return *this;
}

AsciiString AsciiString::operator =(LPCSTR szClone)
{
	ReleaseData();
	if(szClone)
	{
		lpszData = new CHAR[strlen(szClone) + 1];
		if(lpszData)
		{
			memset(lpszData,0,strlen(szClone) + 1);
			memcpy(lpszData,szClone,strlen(szClone));
		}
	}
	return *this;
}

AsciiString AsciiString::operator =(LPSTR szClone)
{
	ReleaseData();
	if(szClone)
	{
		lpszData = new CHAR[strlen(szClone) + 1];
		if(lpszData)
		{
			memset(lpszData,0,strlen(szClone) + 1);
			memcpy(lpszData,szClone,strlen(szClone));
		}
	}
	return *this;
}




AsciiString AsciiString::operator =(CHAR aChar)
{
	ReleaseData();
	this->lpszData = new CHAR[2];
	this->lpszData[0] = aChar;
	this->lpszData[1] = 0;
	return *this;
}


AsciiString AsciiString::operator =(BYTE aChar)
{
	ReleaseData();
	this->lpszData = new CHAR[2];
	this->lpszData[0] = (CHAR) aChar;
	this->lpszData[1] = 0;
	return *this;
}



AsciiString AsciiString::operator =(LPWSTR lpwszClone)
{
	ReleaseData();
	if(lpwszClone)
	{
		lpszData = new CHAR[wcslen(lpwszClone) + 1];
		if(lpszData)
		{
			memset(lpszData,0,wcslen(lpwszClone) + 1);
			WideCharToMultiByte(CP_ACP,NULL,lpwszClone,wcslen(lpwszClone),this->lpszData,wcslen(lpwszClone),NULL,NULL);
		}
	}
	return *this;
}

AsciiString AsciiString::operator =(LPCWSTR lpwszClone)
{
	ReleaseData();
	if(lpwszClone)
	{
		lpszData = new CHAR[wcslen(lpwszClone) + 1];
		if(lpszData)
		{
			memset(lpszData,0,wcslen(lpwszClone) + 1);
			WideCharToMultiByte(CP_ACP,NULL,lpwszClone,wcslen(lpwszClone),this->lpszData,wcslen(lpwszClone),NULL,NULL);
		}
	}
	return *this;
}


AsciiString AsciiString::operator +(AsciiString& AddStr)
{

	ULONG nNewLen = this->length() + AddStr.length();;
	LPSTR lpNewData = NULL;
	// make sure there data in the strings
	if(nNewLen)
	{
		lpNewData = new CHAR[nNewLen + 1];
		memset(lpNewData,0,nNewLen + 1);
		memcpy(lpNewData,this->lpszData,this->length());
		memcpy(lpNewData + this->length(),AddStr.c_str(),AddStr.length());
	}
	ReleaseData();
	this->lpszData = lpNewData;
	return *this;
}




AsciiString AsciiString::operator +(LPSTR AddStr)
{
	ULONG nNewLen = this->length() + strlen(AddStr);
	LPSTR lpNewData = NULL;
	// make sure there data in the strings
	if(nNewLen)
	{
		lpNewData = new CHAR[nNewLen + 1];
		memset(lpNewData,0,nNewLen + 1);
		memcpy(lpNewData,this->lpszData,this->length());
		memcpy(lpNewData + this->length(),AddStr,strlen(AddStr));
	}
	ReleaseData();
	this->lpszData = lpNewData;
	return *this;
}


AsciiString AsciiString::operator +(LPCSTR AddStr)
{
	ULONG nNewLen = this->length() + strlen(AddStr);
	LPSTR lpNewData = NULL;
	// make sure there data in the strings
	if(nNewLen)
	{
		lpNewData = new CHAR[nNewLen + 1];
		memset(lpNewData,0,nNewLen + 1);
		memcpy(lpNewData,this->lpszData,this->length());
		memcpy(lpNewData + this->length(),AddStr,strlen(AddStr));
	}
	ReleaseData();
	this->lpszData = lpNewData;
	return *this;
}


AsciiString AsciiString::operator +(LPWSTR AddStr)
{
	ULONG nNewLen = this->length() + wcslen(AddStr);
	LPSTR lpNewData = NULL;
	// make sure there data in the strings
	if(nNewLen)
	{
		lpNewData = new CHAR[nNewLen + 1];
		memset(lpNewData,0,nNewLen + 1);
		memcpy(lpNewData,this->lpszData,this->length());
		WideCharToMultiByte(CP_ACP,NULL,AddStr,wcslen(AddStr),lpNewData + this->length(),wcslen(AddStr),NULL,NULL);
	}
	ReleaseData();
	this->lpszData = lpNewData;
	return *this;
}


AsciiString AsciiString::operator +(LPCWSTR AddStr)
{
	ULONG nNewLen = this->length() + wcslen(AddStr);
	LPSTR lpNewData = NULL;
	// make sure there data in the strings
	if(nNewLen)
	{
		lpNewData = new CHAR[nNewLen + 1];
		memset(lpNewData,0,nNewLen + 1);
		memcpy(lpNewData,this->lpszData,this->length());
		WideCharToMultiByte(CP_ACP,NULL,AddStr,wcslen(AddStr),lpNewData + this->length(),wcslen(AddStr),NULL,NULL);
	}
	ReleaseData();
	this->lpszData = lpNewData;
	return *this;
}



AsciiString AsciiString::operator +=(AsciiString& AddStr)
{

	ULONG nNewLen = this->length() + AddStr.length();;
	LPSTR lpNewData = NULL;
	// make sure there data in the strings
	if(nNewLen)
	{
		lpNewData = new CHAR[nNewLen + 1];
		memset(lpNewData,0,nNewLen + 1);
		memcpy(lpNewData,this->lpszData,this->length());
		memcpy(lpNewData + this->length(),AddStr.c_str(),AddStr.length());
	}
	ReleaseData();
	this->lpszData = lpNewData;
	return *this;
}




AsciiString AsciiString::operator +=(LPSTR AddStr)
{
	ULONG nNewLen = this->length() + strlen(AddStr);
	LPSTR lpNewData = NULL;
	// make sure there data in the strings
	if(nNewLen)
	{
		lpNewData = new CHAR[nNewLen + 1];
		memset(lpNewData,0,nNewLen + 1);
		memcpy(lpNewData,this->lpszData,this->length());
		memcpy(lpNewData + this->length(),AddStr,strlen(AddStr));
	}
	ReleaseData();
	this->lpszData = lpNewData;
	return *this;
}


AsciiString AsciiString::operator +=(LPCSTR AddStr)
{
	ULONG nNewLen = this->length() + strlen(AddStr);
	LPSTR lpNewData = NULL;
	// make sure there data in the strings
	if(nNewLen)
	{
		lpNewData = new CHAR[nNewLen + 1];
		memset(lpNewData,0,nNewLen + 1);
		memcpy(lpNewData,this->lpszData,this->length());
		memcpy(lpNewData + this->length(),AddStr,strlen(AddStr));
	}
	ReleaseData();
	this->lpszData = lpNewData;
	return *this;
}


AsciiString AsciiString::operator +=(LPWSTR AddStr)
{
	ULONG nNewLen = this->length() + wcslen(AddStr);
	LPSTR lpNewData = NULL;
	// make sure there data in the strings
	if(nNewLen)
	{
		lpNewData = new CHAR[nNewLen + 1];
		memset(lpNewData,0,nNewLen + 1);
		memcpy(lpNewData,this->lpszData,this->length());
		WideCharToMultiByte(CP_ACP,NULL,AddStr,wcslen(AddStr),lpNewData + this->length(),wcslen(AddStr),NULL,NULL);
	}
	ReleaseData();
	this->lpszData = lpNewData;
	return *this;
}


AsciiString AsciiString::operator +=(LPCWSTR AddStr)
{
	ULONG nNewLen = this->length() + wcslen(AddStr);
	LPSTR lpNewData = NULL;
	// make sure there data in the strings
	if(nNewLen)
	{
		lpNewData = new CHAR[nNewLen + 1];
		memset(lpNewData,0,nNewLen + 1);
		memcpy(lpNewData,this->lpszData,this->length());
		WideCharToMultiByte(CP_ACP,NULL,AddStr,wcslen(AddStr),lpNewData + this->length(),wcslen(AddStr),NULL,NULL);
	}
	ReleaseData();
	this->lpszData = lpNewData;
	return *this;
}

CHAR & AsciiString::operator [](ULONG nPos)
{
	if(nPos < length())
	{
		return this->lpszData[nPos];
	}
	return  (CHAR &) this->lpszData;
}

BOOL AsciiString::operator ==(AsciiString &szComp)
{
	if(!strcmp(this->lpszData,szComp.c_str()))
	{
		return TRUE;
	}else
	{
		return FALSE;
	}
}


BOOL AsciiString::operator ==(LPCSTR szComp)
{
	if(!strcmp(this->lpszData,szComp))
	{
		return TRUE;
	}else
	{
		return FALSE;
	}
}

/*
AsciiString AsciiString::operator =(CHAR aChar)
{
	this->lpszData = new CHAR[2];
	this->lpszData[0] = aChar;
	this->lpszData[1] = 0;
	return *this;
}
*/

AsciiString AsciiString::operator =(WCHAR aChar)
{
	this->lpszData = new CHAR[2];
	this->lpszData[0] = (CHAR) aChar;
	this->lpszData[1] = 0;
	return *this;
}


AsciiString AsciiString::operator +=(CHAR aChar)
{
	ULONG nNewLen = this->length() + 1;
	LPSTR lpNewData = NULL;
	// make sure there data in the strings
	if(nNewLen)
	{
		lpNewData = new CHAR[nNewLen + 1];
		memset(lpNewData,0,nNewLen + 1);
		memcpy(lpNewData,this->lpszData,this->length());
		lpNewData[length()] = aChar;
	}
	ReleaseData();
	this->lpszData = lpNewData;
	return *this;
}


AsciiString AsciiString::operator +=(BYTE aChar)
{
	ULONG nNewLen = this->length() + 1;
	LPSTR lpNewData = NULL;
	// make sure there data in the strings
	if(nNewLen)
	{
		lpNewData = new CHAR[nNewLen + 1];
		memset(lpNewData,0,nNewLen + 1);
		memcpy(lpNewData,this->lpszData,this->length());
		lpNewData[length()] = (BYTE) aChar;
	}
	ReleaseData();
	this->lpszData = lpNewData;
	return *this;
}


AsciiString AsciiString::operator +=(WCHAR aChar)
{
	ULONG nNewLen = this->length() + 1;
	LPSTR lpNewData = NULL;
	// make sure there data in the strings
	if(nNewLen)
	{
		lpNewData = new CHAR[nNewLen + 1];
		memset(lpNewData,0,nNewLen + 1);
		memcpy(lpNewData,this->lpszData,this->length());
		lpNewData[length()] =  (CHAR) aChar;
	}
	ReleaseData();
	this->lpszData = lpNewData;
	return *this;
}


// self describing code
AsciiString AsciiString::substr(int pos, int len)
{
	AsciiString Ret;
	int nEnd;
	LPSTR Buff;
	if(pos < length())
	{
		if(len == -1 || (( pos + len) > length()) )
		{
			nEnd = length() - pos;
		}else
		{
			nEnd = len;
		}
		Buff = new CHAR((nEnd - pos) + 1);
		memset(Buff,0,(nEnd - pos) + 1);
		memcpy(Buff,this->lpszData + pos,(nEnd - pos));
		Ret = Buff;
		delete Buff;
		Buff = NULL;

	}
	return Ret;

}

// self describing code
int AsciiString::find(LPCSTR szFind)
{
   int nIndex;
   int nMax;
   int nsLen;
   if(szFind)
   {
		nMax = length();
		nsLen = strlen(szFind);
		for(nIndex = 0;nIndex < (nMax - nsLen) ;nIndex++)
		{
			if(!strncmp(this->lpszData+nIndex,szFind,nsLen))
			{
				return nIndex;
			}
		}
   }

   return -1;
}


// CTime under WCE dont have a format command so I wrote this simple BF format routine
// it returns back a CString representing the date passes in NO there are no format spsifiers
// yoiu get the date the way I give it to you if this dose not satisfy go else where
CString BFGetStrDate(CTime *pDate)
{
	CString sRet;
	CString Days[] = {"Sun","Mon","Tue","Wen","Thu","Fri","Sat"};
	CString Months[] = {"Jan","Feb","Mar","Apr","May","Jun","Jul",
		"Aug","Sep","Oct","Nov","Dec"};
	sRet.Format(TEXT("%s, %d %s %d %d:%02d:%02d"),
		Days[pDate->GetDayOfWeek() - 1],pDate->GetDay(),Months[pDate->GetMonth() - 1],
		pDate->GetYear(),pDate->GetHour(),pDate->GetMinute(),pDate->GetSecond());

	return sRet;
}
